
/*
---------------------------------------------------------------------------
ECN FreeSoft
---------------------------------------------------------------------------
Progetto : music32
Modulo   : device.h
Inizio   : 29.07.2000
Fine     :
Versione : 1.0
Autore/i : Giorgio D.Plescan
Commento :
---------------------------------------------------------------------------
*/

/*
---------------------------------------------------------------------------
Define
---------------------------------------------------------------------------
*/

/**/
# define        PI_GRECO		3.1415926535
# define        _2PI_GRECO		6.2831853069

# define        F_CAMPION		44100.0
# define        F_CAMPION_2		22050.0
# define	PUNTI_VIRTUALI		4000000.0
# define	PUNTI_VIRTUALI_2	2000000.0

# define        N_PUNTI			8192
# define        N_PUNTI_2		4096

/**/
# define        N_ONDE			6

# define        SENO			0
# define        TRIANG			1
# define        QUADRA			2
# define        RAMPA			3
# define        PARAB			4
# define        TANG			5

# define        DISPOSITIVO		0
# define        SOMMATORE		1
# define        RUMORE			2
# define        OSCILLATORE		3

/**/
# define	DEV_NAME_LEN		32

/*
---------------------------------------------------------------------------
Class
---------------------------------------------------------------------------
*/

class Segnale
{

	private:

		double
			Valore;

	public:

		Segnale (void);
		Segnale (double val);

		double LeggiValore  (void);
		void   ScriviValore (double val);

		Segnale operator+(Segnale val);
		Segnale operator-(Segnale val);
		Segnale operator-(void);

		Segnale operator*(Segnale val);
		Segnale operator/(Segnale val);
		Segnale operator=(Segnale val);

		int operator==(Segnale val);
		int operator!=(Segnale val);
		int operator<(Segnale val);
		int operator>(Segnale val);
		int operator<=(Segnale val);
		int operator>=(Segnale val);

};

class Formaonda
{

	private:

		double
			inc_sin,
			inc_rmp,
			inc_trg,	
			inc_reali,
			Campione [N_ONDE][N_PUNTI];

	public:

		Formaonda (void);

		void   Init           (void);
		double LeggiCampione  (int on, double idx);

};

class Dispositivo
{

	private:

		Dispositivo
			** p_Ingresso;

		DWORD
			Chiamata;

	protected:

		char
			Nome [DEV_NAME_LEN + 1];

		int
			* IndiceUscite,
			n_Ingressi,
			n_Uscite;

		Segnale
			** Uscita,
			** Ingresso;

		void Init (int ing, int usc);

	public:

		Dispositivo (void);
		Dispositivo (int ing, int usc);

		void  Debug           (void);

		DWORD LeggiChiamata    (void);
		void  LeggiNome        (char * str);
		void  Elabora          (DWORD chiamata);
		void  ScriviChiamata   (DWORD chiamata);
		void  ScriviNome       (char * str);
		void  ScriviUscita     (int canale, Segnale val);
		void  ScriviIngresso   (int canale, Segnale val);
		void  CollegaIngresso  (int canale_src, int canale_dst, Dispositivo * indirizzo);
		void  StaccaIngresso   (int canale);
		void  AggiornaIngresso (int canale);
		
		int  NumeroIngressi   (void);
		int  NumeroUscite     (void);
		int  IngressoAttivo   (int canale);

		Segnale LeggiUscita   (int canale);
		Segnale LeggiIngresso (int canale);

		Dispositivo * Esporta (void);

		virtual void Custom   (void);
		virtual void Calcola  (void);

};

class Registratore : public Dispositivo
{

	private:

		char
			NomeFile [128];
		int
			Piste;
		FILE
			* Fp;

	public:

		Registratore (void);
		Registratore (char * nomef, int piste);

		void Apri   (void);
		void Scrivi (void);
		void Chiudi (void);

		static void ChiudiTutto (void);

		void Custom  (void);
		void Calcola (void);


};

class Rumore : public Dispositivo
{

	private:

		double
			inc_rnd;

	public:

		Rumore (void);

		void Custom  (void);
		void Calcola (void);

};

class Conduttore : public Dispositivo
{

	public:

		Conduttore (void);

		void Custom  (void);
		void Calcola (void);

};

class Invertitore : public Dispositivo
{

	public:

		Invertitore (void);

		void Custom  (void);
		void Calcola (void);

};

class Assoluto : public Dispositivo
{

	public:

		Assoluto (void);

		void Custom  (void);
		void Calcola (void);

};

class Memoria : public Dispositivo
{

	private:

		Segnale
			Mem;

	public:

		Memoria (void);
		Memoria (Segnale val);

		Segnale LeggiMemoria  (void);
		void    ScriviMemoria (Segnale val);

		void Custom  (void);
		void Calcola (void);

};

class Sommatore : public Dispositivo
{

	public:
	
		Sommatore (void);
		Sommatore (int nin);

		void Custom  (void);
		void Calcola (void);

};

class Moltiplicatore : public Dispositivo
{

	public:

		Moltiplicatore (void);
		Moltiplicatore (int nin);

		void Custom  (void);
		void Calcola (void);

};

class Modulatore : public Dispositivo
{

	public:

		Modulatore   (void);

		void Custom  (void);
		void Calcola (void);

};

class Compressore : public Dispositivo
{

	private:

		Segnale
			Soglia;

	public:

		Compressore (void);
		Compressore (double val);

		double LeggiSoglia  (void);
		void   ScriviSoglia (double val);

		void Custom  (void);
		void Calcola (void);

};

class Oscillatore: public Dispositivo
{

	private:

		int
			Onda;		// Forma onda

		double
			inc_virtuali,
			inc_fase,
			Cont,
			Duty,		// Contatore tabella campioni
			Fase,		// Fase
			Freq;		// Frequenza

		static Formaonda
			* Tabella;

	public:

		Oscillatore (int onda, double freq);
		Oscillatore (void);

		void ScriviCont (double cont);
		void ScriviOnda (int onda);

		double LeggiCont (void);
		int    LeggiOnda (void);

		void ScriviFrequenza (double freq, double mfreq);
		void ScriviFase      (double fase);
		void ScriviDuty      (double duty);

		double LeggiFrequenza (void);
		double LeggiFase      (void);
		double LeggiDuty      (void);

		void Custom  (void);
		void Calcola (void);

};

class Sintetizzatore
{

		private:

			int
				Quanti;
			DWORD
				Contatore;

			Dispositivo
				* Lista [1024];

		public:

			Sintetizzatore (void);
			Sintetizzatore (int ndevice);

			int  CollegaOggetti   (int src, int cs, int dest, int cnl);
			int  IngressoOggetti  (double value, int dest, int cnl);
			void CancellaOggetti  (void);
			void InserisciOggetti (Dispositivo * dev, char * nome);
			int  CercaOggetti     (char * nome);
			void ElaboraOggetti   (void);

			void DebugOggetti     (void);

};

/*
---------------------------------------------------------------------------
Typedef
---------------------------------------------------------------------------
*/

typedef union
{

	Dispositivo    * P_dis;
	Sommatore      * P_som;
	Rumore         * P_rum;
	Oscillatore    * P_osc;
	Moltiplicatore * P_mol;
	Memoria        * P_mem;
	Assoluto       * P_ass;
	Conduttore     * P_con;
	Invertitore    * P_inv;
	Compressore    * P_com;

} Referenza;
